<?php

namespace Myriad\Authentication;

use Illuminate\Auth\AuthManager;
use Illuminate\Contracts\Hashing\Hasher;
use Illuminate\Contracts\Validation\Factory as ValidationFactory;
use Illuminate\Database\Capsule\Manager as Capsule;
use Illuminate\Database\Eloquent\Relations\Relation;
use Illuminate\Events\Dispatcher;
use Illuminate\Support\ServiceProvider;
use Myriad\Authentication\Http\Transformers\IUserTransformer;
use Myriad\Authentication\Lib\Guards\AccessTokenGuard;
use Myriad\Authentication\Lib\Providers\EloquentUserProvider;
use Myriad\Authentication\Lib\Repositories\EloquentAccessTokenRepository;
use Myriad\Authentication\Lib\Repositories\EloquentVerificationCodeRepository;
use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;
use Myriad\Authentication\Lib\Repositories\IUserRepository;
use Myriad\Authentication\Lib\Repositories\IVerificationCodeRepository;
use Myriad\Authentication\Lib\Services\IEmailConfirmer;
use Myriad\Authentication\Lib\Services\IPhoneNumberConfirmer;
use Myriad\Authentication\Lib\Services\ITextMessageSender;
use Myriad\Authentication\Lib\Services\MockTextMessageSender;
use Myriad\Authentication\Lib\Strategies\EmailVerification;
use Myriad\Authentication\Lib\Strategies\PhoneNumberVerification;
use Myriad\Installations\InstallationManager;
use PDO;

/**
 * Class: AuthenticationServiceProvider
 *
 * @see ServiceProvider
 */
class AuthenticationServiceProvider extends ServiceProvider
{

    /**
     * @return void
     */
    public function boot(): void
    {
        $this->globalizeCapsule();
        $this->bindRoutes();
        $this->publishAssets();
        $this->loadViewsFrom(__DIR__.'/../resources/views', 'authentication');
        $this->loadTranslationsFrom(__DIR__.'/../resources/lang', 'unique_verified');
        $this->appendNewRulesToValidator();
        $this->configureModelMorphMap();
        $this->bindAuthManager();
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        app()->bind(IUserRepository::class, config('authentication.user_repository'));
        app()->bind(IUserTransformer::class, config('authentication.user_transformer'));
        app()->bind(IAccessTokenRepository::class, EloquentAccessTokenRepository::class);
        app()->bind(IVerificationCodeRepository::class, EloquentVerificationCodeRepository::class);

        app()->bind(
            IEmailConfirmer::class,
            config('authentication.verification_strategies.email', EmailVerification::class)
        );

        app()->bind(
            IPhoneNumberConfirmer::class,
            config('authentication.verification_strategies.phone', PhoneNumberVerification::class)
        );

        if (config('app.env') === 'testing') {
            app()->bind(ITextMessageSender::class, MockTextMessageSender::class);
        } else {
            app()->bind(
                ITextMessageSender::class,
                config('authentication.text_messages.text_message_sender.class')
            );
        }
    }

    /**
     * Enable Eloquent interaction
     */
    private function globalizeCapsule(): void
    {
        $defaultDatabase = config('database.default');
        // Newing up a Capsule environment will overwrite the
        // fetch type and connection details
        $capsule = app(Capsule::class);
        config(['database.default' => $defaultDatabase]);
        $capsule->setFetchMode(PDO::FETCH_CLASS);
        $connection = config('database.connections.'.config('database.default'));
        $capsule->addConnection($connection);
        $capsule->setAsGlobal();
    }

    /**
     * Include default authentication routes
     */
    private function bindRoutes(): void
    {
        if (!$this->app->routesAreCached()) {
            require __DIR__.'/routes.php';
        }
    }

    /**
     * Registers all paths to be published
     */
    private function publishAssets(): void
    {
        $this->publishes([
            __DIR__.'/../database/migrations/' => database_path('migrations'),
        ], 'migrations');

        $this->publishes([
            __DIR__.'/../config/authentication.php' => config_path('authentication.php'),
        ], 'config');
    }

    /**
     * Registers an auth provider which will resolve users via AccessTokens
     */
    private function bindAuthManager(): void
    {
        /** @var AuthManager $auth */
        $auth = app(AuthManager::class);

        $auth->provider('myriad_user_provider', function ($app, array $config) {
            return new EloquentUserProvider(
                app(Hasher::class),
                app(IAccessTokenRepository::class),
                $config['model']
            );
        });

        $auth->extend('myriad_access_token_guard', function ($app, $name, array $config) use ($auth) {
            return new AccessTokenGuard(
                $auth->createUserProvider($config['provider']),
                $app['request'],
                'token',
                'token'
            );
        });
    }

    /**
     * Adds the unique_verified validation rule to the validation system
     */
    private function appendNewRulesToValidator(): void
    {
        app(ValidationFactory::class)->extend(
            'unique_verified',
            function ($attribute, $value, $parameters, $validator) {
                $parameters = array_map('trim', $parameters);
                $table = $parameters[0];
                $column = $attribute;

                $verifier = $validator->getPresenceVerifier();
                $count = $verifier->getCount(
                    $table,
                    $column,
                    $value,
                    null,
                    null,
                    ['verified' => true]
                );

                return $count === 0;
            }
        );
    }

    /**
     * Configure the morphMap for model relationships
     */
    private function configureModelMorphMap(): void
    {
        $morphMap = config('authentication.morph_map');

        if (!is_null($morphMap)) {
            Relation::morphMap($morphMap);
        }
    }
}
