<?php

namespace Myriad\Authentication\Lib\Providers;

use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;
use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Auth\EloquentUserProvider as Base;
use Illuminate\Contracts\Hashing\Hasher;

class EloquentUserProvider extends Base
{
  /**
   * The access token repository
   *
   * @var IAccessTokenRepository
   */
  protected $accessTokenRepository;

  /**
   * Create a new database user provider.
   *
   * @param  Hasher  $hasher
   * @param  IAccessTokenRepository  $accessTokenRepository
   * @param  string  $model
   * @return void
   */
  public function __construct(
    Hasher $hasher,
    IAccessTokenRepository $accessTokenRepository,
    $model)
  {
    parent::__construct($hasher, $model);

    $this->accessTokenRepository = $accessTokenRepository;
  }

  /**
   * Retrieve a user by the given credentials.
   *
   * @param  array  $credentials
   * @return \Illuminate\Contracts\Auth\Authenticatable|null
   */
  public function retrieveByCredentials(array $credentials) {
    $user = $this->retrieveByAccessToken($credentials);

    if (is_null($user)) {
      $user = $this->createModel()->lookup(array_diff_key($credentials, array_flip(['password'])));
    }

    return $user;
  }

  /**
   * Validate a user against the given credentials.
   *
   * @param  \Illuminate\Contracts\Auth\Authenticatable  $user
   * @param  array  $credentials
   * @return bool
   */
  public function validateCredentials(Authenticatable $user, array $credentials) {
    $valid = $this->validatePasswordCredentials($user, $credentials);

    if (!$valid) {
      $valid = $this->validateTokenCredentials($user, $credentials);
    }

    return $valid;
  }


  protected function retrieveByAccessToken(array $credentials) {
    if (!$this->containsToken($credentials)) {
      return null;
    }

    return $this->accessTokenRepository->getUserFromToken($credentials['token']);
  }

  /**
   * Is there a non-null password in credentials array
   *
   * @param array $credentials
   */
  protected function containsPassword(array $credentials) {
    return $this->credentialsContainKey($credentials, 'password');
  }

  /**
   * Is there a non-null token in the credentials array
   *
   * @param array $credentials
   */
  protected function containsToken(array $credentials) {
    return $this->credentialsContainKey($credentials, 'token');
  }


  /**
   * Do the credentials contain a non-null value for key
   *
   * @param array $credentials
   * @param mixed $key
   */
  protected function credentialsContainKey(array $credentials, $key) {
    if (empty($credentials)) {
        return false;
    }

    if (!array_key_exists($key, $credentials) || is_null($credentials[$key])) {
      return false;
    }

    return true;
  }


  /**
   * Validate the user against credentials by checking password
   *
   * @param Authenticatable $user
   * @param mixed $credentials
   */
  protected function validatePasswordCredentials(Authenticatable $user, $credentials) {
    if (!$this->containsPassword($credentials)) {
      return false;
    }

    $plain = $credentials['password'];

    return $this->hasher->check($plain, $user->getAuthPassword());
  }

  /**
   * Validate the user against credentials by checking if there is a valid token
   *
   * @param Authenticatable $user
   * @param mixed $credentials
   */
  protected function validateTokenCredentials(Authenticatable $user, $credentials) {
    // make sure there's a token in the credentials
    if (!$this->containsToken($credentials)) {
      return false;
    }

    // check if token exists
    $token = $this->accessTokenRepository->getAccessTokenByToken($credentials['token']);
    if (is_null($token)) {
      return false;
    }

    // check that is is not expired
    if ($token->isExpired()) {
      return false;
    }

    // lastly, make sure the token is for the user
    return $token->isForUser($user);
  }

}
