<?php

namespace Myriad\Authentication\Lib\Repositories;

use Carbon\Carbon;
use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Support\Facades\DB;
use Myriad\Authentication\Models\AccessToken;
use Myriad\Installations\Models\Installation;

class EloquentAccessTokenRepository implements IAccessTokenRepository
{

    /**
     * @param  Authenticatable  $user
     * @param  Installation  $installationClient
     * @return mixed
     */
    public function createAccessToken($user, $installationClient)
    {
        $token = $this->generateToken();

        $tokenLifetime = config('authentication.access_token_expiration_days', 365);
        $expiresAt = Carbon::now()->addDays($tokenLifetime);

        return AccessToken::create([
            'token' => $token,
            'expires_at' => $expiresAt,
            'owner_type' => $user->getMorphClass(),
            'owner_id' => $user->id,
            'client_type' => $installationClient->getMorphClass(),
            'client_id' => $installationClient->id,
        ]);
    }

    /**
     * @param  string  $token
     * @return Authenticatable|null
     */
    public function getUserFromToken(string $token): Authenticatable
    {
        $accessToken = $this->getAccessTokenByToken($token);

        if (is_null($accessToken)) {
            return null;
        }

        return $accessToken->owner;
    }

    public function getAccessTokenByToken($token, $expired = false): ?AccessToken
    {
        $query = AccessToken::where('token', $token);
        if ($expired === false) {
            $query->where('expires_at', '>', DB::raw('now()'));
        } else {
            if ($expired === true) {
                $query->where('expires_at', '<=', DB::raw('now()'));
            }
        }
        return $query->first();
    }

    public function logout($user, $installationClient, $all): void
    {
        $query = AccessToken::query()
            ->where('owner_id', $user->id)
            ->where('owner_type', $user->getMorphClass())
            ->where('client_type', $installationClient->getMorphClass());
        // only log out our current user if all is not sent or is false
        if (!$all || $all === 'false') {
            $query->where('client_id', $installationClient->id);
        }

        $query->update(['expires_at' => DB::raw('now()')]);
    }

    private function generateToken($bytes = 96)
    {
        $rand = random_bytes($bytes);
        $rand = base64_encode($rand);
        $rand = str_replace(['/', '+', '='], ['_', '-', '*'], $rand); // replace non-url safe characters
        return $rand;
    }
}
